"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ControlPlugin = void 0;

var _tslib = require("tslib");

var _g = require("@antv/g");

var _manaSyringe = require("mana-syringe");

var _hammerjs = _interopRequireDefault(require("hammerjs"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var MOTION_FACTOR = 10; // https://gist.github.com/handleman/3c99e754065f647b082f

var isMac = navigator.platform.toUpperCase().indexOf('MAC') >= 0;

var ControlPlugin =
/** @class */
function () {
  function ControlPlugin() {
    var _this = this;

    this.isMoving = false;
    this.lastX = -1;
    this.lastY = -1;

    this.onPanstart = function (e) {
      _this.lastX = e.center.x;
      _this.lastY = e.center.y;
      _this.isMoving = true;
    };

    this.onPanmove = function (e) {
      _this.ctrlKey = e.srcEvent.ctrlKey;

      if (isMac && e.srcEvent.metaKey) {
        _this.ctrlKey = true;
      }

      _this.altKey = e.srcEvent.altKey;
      _this.shiftKey = e.srcEvent.shiftKey;

      if (_this.isMoving) {
        var deltaX = e.center.x - _this.lastX;
        var deltaY = e.center.y - _this.lastY;
        _this.lastX = e.center.x;
        _this.lastY = e.center.y;

        if (_this.ctrlKey && !_this.shiftKey) {
          _this.dolly(deltaY);
        } else if (_this.shiftKey && !_this.ctrlKey) {
          _this.pan(deltaX, deltaY);
        } else if (_this.ctrlKey && _this.shiftKey) {
          _this.roll(deltaY);
        } else {
          _this.rotate(deltaX, deltaY);
        }
      }
    };

    this.onPanend = function (e) {
      _this.isMoving = false;
    };

    this.onPinch = function (e) {
      var deltaZ = (1 - e.scale) * 10;

      _this.dolly(deltaZ);
    };

    this.onMousewheel = function (e) {
      _this.dolly(e.deltaY);
    };
  }

  ControlPlugin.prototype.apply = function (renderingService) {
    var _this = this;

    renderingService.hooks.init.tapPromise(function () {
      return (0, _tslib.__awaiter)(_this, void 0, void 0, function () {
        var root;
        return (0, _tslib.__generator)(this, function (_a) {
          root = this.renderingContext.root.ownerDocument.defaultView; // @ts-ignore

          this.hammertime = new _hammerjs.default(root);
          this.hammertime.get('pan').set({
            direction: _hammerjs.default.DIRECTION_ALL
          });
          this.hammertime.get('pinch').set({
            enable: true
          });
          this.hammertime.on('panstart', this.onPanstart);
          this.hammertime.on('panmove', this.onPanmove);
          this.hammertime.on('panend', this.onPanend);
          this.hammertime.on('pinch', this.onPinch);
          root.addEventListener('wheel', this.onMousewheel);
          return [2
          /*return*/
          ];
        });
      });
    });
    renderingService.hooks.destroy.tap(function () {
      _this.hammertime.off('panstart', _this.onPanstart);

      _this.hammertime.off('panmove', _this.onPanmove);

      _this.hammertime.off('panend', _this.onPanend);

      _this.hammertime.off('pinch', _this.onPinch);

      var root = _this.renderingContext.root;
      root.removeEventListener('wheel', _this.onMousewheel);
    });
  };

  ControlPlugin.prototype.dolly = function (z) {
    this.camera.dolly(z);
  };

  ControlPlugin.prototype.pan = function (dx, dy) {
    var _a = this.canvasConfig,
        width = _a.width,
        height = _a.height;
    var dimMax = Math.max(width, height);
    var deltaX = 1 / dimMax;
    var deltaY = 1 / dimMax;
    var ndx = dx * deltaX * MOTION_FACTOR / 2;
    var ndy = -dy * deltaY * MOTION_FACTOR / 2;
    this.camera.pan(ndx, ndy);
  };

  ControlPlugin.prototype.roll = function (dy) {
    var width = this.canvasConfig.width;
    this.camera.rotate(0, 0, -20.0 / width * dy * MOTION_FACTOR);
  };

  ControlPlugin.prototype.rotate = function (rx, ry) {
    var _a = this.canvasConfig,
        width = _a.width,
        height = _a.height;
    var dx = 20.0 / height;
    var dy = 20.0 / width;
    var motionFactorX = MOTION_FACTOR;
    var motionFactorY = MOTION_FACTOR;

    if (rx * rx > 2 * ry * ry) {
      motionFactorY *= 0.5;
    } else if (ry * ry > 2 * rx * rx) {
      motionFactorX *= 0.5;
    }

    var rotX = rx * dx * motionFactorX;
    var rotY = ry * dy * motionFactorY;
    this.camera.rotate(rotX, -rotY, 0);
  };

  (0, _tslib.__decorate)([(0, _manaSyringe.inject)(_g.RenderingContext), (0, _tslib.__metadata)("design:type", Object)], ControlPlugin.prototype, "renderingContext", void 0);
  (0, _tslib.__decorate)([(0, _manaSyringe.inject)(_g.DefaultCamera), (0, _tslib.__metadata)("design:type", _g.Camera)], ControlPlugin.prototype, "camera", void 0);
  (0, _tslib.__decorate)([(0, _manaSyringe.inject)(_g.CanvasConfig), (0, _tslib.__metadata)("design:type", Object)], ControlPlugin.prototype, "canvasConfig", void 0);
  ControlPlugin = (0, _tslib.__decorate)([(0, _manaSyringe.singleton)({
    contrib: _g.RenderingPluginContribution
  })], ControlPlugin);
  return ControlPlugin;
}();

exports.ControlPlugin = ControlPlugin;